@@ -5,7 +5,7 @@ import warnings
 from astropy.io import fits
 from astropy.wcs import WCS
 from astropy.extern import six
-from astropy.utils import OrderedDict
+from collections import OrderedDict
 from astropy.io.fits.hdu.hdulist import fitsopen as fits_open
 
 import numpy as np
@@ -17,7 +17,8 @@ except ImportError:
     # We might be running py.test on a clean checkout
     SPECTRAL_CUBE_VERSION = 'dev'
 
-from .. import SpectralCube, StokesSpectralCube, LazyMask
+from .. import SpectralCube, StokesSpectralCube, LazyMask, VaryingResolutionSpectralCube
+from ..spectral_cube import BaseSpectralCube
 from .. import cube_utils
 
 
@@ -44,7 +45,7 @@ def is_fits(input, **kwargs):
         return False
 
 
-def read_data_fits(input, hdu=None, **kwargs):
+def read_data_fits(input, hdu=None, mode='denywrite', **kwargs):
     """
     Read an array and header from an FITS file.
 
@@ -58,8 +59,15 @@ def read_data_fits(input, hdu=None, **kwargs):
         - :class:`~astropy.io.fits.hdu.hdulist.HDUList`
     hdu : int or str, optional
         The HDU to read the table from.
+    mode : str
+        One of the FITS file reading modes; see `~astropy.io.fits.open`.
+        ``denywrite`` is used by default since this prevents the system from
+        checking that the entire cube will fit into swap, which can prevent the
+        file from being opened at all.
     """
 
+    beam_table = None
+
     if isinstance(input, fits.HDUList):
 
         # Parse all array objects
@@ -67,6 +75,9 @@ def read_data_fits(input, hdu=None, **kwargs):
         for ihdu, hdu_item in enumerate(input):
             if isinstance(hdu_item, (fits.PrimaryHDU, fits.ImageHDU)):
                 arrays[ihdu] = hdu_item
+            elif isinstance(hdu_item, fits.BinTableHDU):
+                if 'BPA' in hdu_item.data.names:
+                    beam_table = hdu_item.data
 
         if len(arrays) > 1:
             if hdu is None:
@@ -95,14 +106,14 @@ def read_data_fits(input, hdu=None, **kwargs):
 
     else:
 
-        hdulist = fits_open(input, **kwargs)
+        hdulist = fits_open(input, mode=mode, **kwargs)
 
         try:
             return read_data_fits(hdulist, hdu=hdu)
         finally:
             hdulist.close()
 
-    return array_hdu.data, array_hdu.header
+    return array_hdu.data, array_hdu.header, beam_table
 
 
 def load_fits_cube(input, hdu=0, meta=None, **kwargs):
@@ -119,7 +130,7 @@ def load_fits_cube(input, hdu=0, meta=None, **kwargs):
         Metadata (can be inherited from other readers, for example)
     """
 
-    data, header = read_data_fits(input, hdu=hdu, **kwargs)
+    data, header, beam_table = read_data_fits(input, hdu=hdu, **kwargs)
 
     if meta is None:
         meta = {}
@@ -135,8 +146,18 @@ def load_fits_cube(input, hdu=0, meta=None, **kwargs):
 
         mask = LazyMask(np.isfinite, data=data, wcs=wcs)
         assert data.shape == mask._data.shape
-        cube = SpectralCube(data, wcs, mask, meta=meta, header=header)
-        assert cube._data.shape == cube._mask._data.shape
+        if beam_table is None:
+            cube = SpectralCube(data, wcs, mask, meta=meta, header=header)
+        else:
+            cube = VaryingResolutionSpectralCube(data, wcs, mask, meta=meta,
+                                                 header=header,
+                                                 beam_table=beam_table)
+        
+        if hasattr(cube._mask, '_data'):
+            # check that the shape matches if there is a shape
+            # it is possible that VaryingResolution cubes will have a composite
+            # mask instead
+            assert cube._data.shape == cube._mask._data.shape
 
     elif wcs.wcs.naxis == 4:
 
@@ -146,9 +167,16 @@ def load_fits_cube(input, hdu=0, meta=None, **kwargs):
         for component in data:
             comp_data, comp_wcs = cube_utils._orient(data[component], wcs)
             comp_mask = LazyMask(np.isfinite, data=comp_data, wcs=comp_wcs)
-            stokes_data[component] = SpectralCube(comp_data, wcs=comp_wcs,
-                                                  mask=comp_mask, meta=meta,
-                                                  header=header)
+            if beam_table is None:
+                stokes_data[component] = SpectralCube(comp_data, wcs=comp_wcs,
+                                                      mask=comp_mask, meta=meta,
+                                                      header=header)
+            else:
+                VRSC = VaryingResolutionSpectralCube
+                stokes_data[component] = VRSC(comp_data, wcs=comp_wcs,
+                                              mask=comp_mask, meta=meta,
+                                              header=header,
+                                              beam_table=beam_table)
 
         cube = StokesSpectralCube(stokes_data)
 
@@ -165,13 +193,13 @@ def write_fits_cube(filename, cube, overwrite=False,
     Write a FITS cube with a WCS to a filename
     """
 
-    if isinstance(cube, SpectralCube):
-        hdu = cube.hdu
+    if isinstance(cube, BaseSpectralCube):
+        hdulist = cube.hdulist
         now = datetime.datetime.strftime(datetime.datetime.now(),
                                          "%Y/%m/%d-%H:%M:%S")
-        hdu.header.add_history("Written by spectral_cube v{version} on "
-                               "{date}".format(version=SPECTRAL_CUBE_VERSION,
-                                               date=now))
-        hdu.writeto(filename, clobber=overwrite)
+        hdulist[0].header.add_history("Written by spectral_cube v{version} on "
+                                      "{date}".format(version=SPECTRAL_CUBE_VERSION,
+                                                      date=now))
+        hdulist.writeto(filename, clobber=overwrite)
     else:
         raise NotImplementedError()
