@@ -1,30 +1,23 @@
 from __future__ import print_function, absolute_import, division
 
+import warnings
 from astropy import units as u
 from astropy import wcs
-from astropy.io.fits import PrimaryHDU, ImageHDU, Header, Card, HDUList
-from astropy import wcs
+from astropy.io.fits import Header, Card, HDUList, PrimaryHDU
 from .io.core import determine_format
+from . import spectral_axis
 
 import numpy as np
 
-class LowerDimensionalObject(u.Quantity):
+from .base_class import BaseNDClass, SpectralAxisMixinClass
+from .cube_utils import beams_to_bintable
+
+
+class LowerDimensionalObject(u.Quantity, BaseNDClass):
     """
     Generic class for 1D and 2D objects
     """
 
-    @property
-    def wcs(self):
-        return self._wcs
-
-    @property
-    def meta(self):
-        return self._meta
-
-    @property
-    def mask(self):
-        return self._mask
-
     @property
     def header(self):
         header = self._header
@@ -33,19 +26,25 @@ class LowerDimensionalObject(u.Quantity):
         if self.wcs is not None:
             header.update(self.wcs.to_header())
         header['BUNIT'] = self.unit.to_string(format='fits')
+        for keyword in header:
+            if 'NAXIS' in keyword:
+                del header[keyword]
         header.insert(2, Card(keyword='NAXIS', value=self.ndim))
         for ind,sh in enumerate(self.shape[::-1]):
             header.insert(3+ind, Card(keyword='NAXIS{0:1d}'.format(ind+1),
                                       value=sh))
+
+        if 'beam' in self.meta:
+            header.update(self.meta['beam'].to_header_keywords())
+
         return header
 
     @property
     def hdu(self):
-        from astropy.io import fits
         if self.wcs is None:
-            hdu = fits.PrimaryHDU(self.value)
+            hdu = PrimaryHDU(self.value)
         else:
-            hdu = fits.PrimaryHDU(self.value, header=self.wcs.to_header())
+            hdu = PrimaryHDU(self.value, header=self.wcs.to_header())
         hdu.header['BUNIT'] = self.unit.to_string(format='fits')
 
         if 'beam' in self.meta:
@@ -92,7 +91,7 @@ class LowerDimensionalObject(u.Quantity):
 
         return new
 
-    def __getitem__(self, key):
+    def __getitem__(self, key, **kwargs):
         """
         Return a new ``LowerDimensionalObject'' of the same class while keeping
         other properties fixed.
@@ -114,16 +113,17 @@ class LowerDimensionalObject(u.Quantity):
                              wcs=newwcs,
                              meta=self._meta,
                              mask=self._mask,
-                             header=self._header)
+                             header=self._header,
+                             **kwargs)
 
         return new
 
     def __array_finalize__(self, obj):
-        self._unit = getattr(obj, '_unit', None)
         self._wcs = getattr(obj, '_wcs', None)
         self._meta = getattr(obj, '_meta', None)
         self._mask = getattr(obj, '_mask', None)
         self._header = getattr(obj, '_header', None)
+        super(LowerDimensionalObject, self).__array_finalize__(obj)
 
     @property
     def __array_priority__(self):
@@ -193,10 +193,11 @@ class Slice(Projection):
     pass
 
 
-class OneDSpectrum(LowerDimensionalObject):
+class OneDSpectrum(LowerDimensionalObject,SpectralAxisMixinClass):
 
     def __new__(cls, value, unit=None, dtype=None, copy=True, wcs=None,
-                meta=None, mask=None, header=None):
+                meta=None, mask=None, header=None, spectral_unit=None,
+                beams=None):
 
         if np.asarray(value).ndim != 1:
             raise ValueError("value should be a 1-d array")
@@ -214,15 +215,59 @@ class OneDSpectrum(LowerDimensionalObject):
         else:
             self._header = Header()
 
+        self._spectral_unit = spectral_unit
+
+        if spectral_unit is None:
+            if 'CUNIT1' in self._header:
+                self._spectral_unit = u.Unit(self._header['CUNIT1'])
+            elif self._wcs is not None:
+                self._spectral_unit = u.Unit(self._wcs.wcs.cunit[0])
+
+        if beams is not None:
+            self.beams = beams
+
         return self
 
+
+    @property
+    def header(self):
+        header = self._header
+        # This inplace update is OK; it's not bad to overwrite WCS in this
+        # header
+        if self.wcs is not None:
+            header.update(self.wcs.to_header())
+        header['BUNIT'] = self.unit.to_string(format='fits')
+        header.insert(2, Card(keyword='NAXIS', value=self.ndim))
+        for ind,sh in enumerate(self.shape[::-1]):
+            header.insert(3+ind, Card(keyword='NAXIS{0:1d}'.format(ind+1),
+                                      value=sh))
+
+        # Preserve the spectrum's spectral units
+        if self._spectral_unit != u.Unit(header['CUNIT1']):
+            spectral_scale = spectral_axis.wcs_unit_scale(self._spectral_unit)
+            header['CDELT1'] *= spectral_scale
+            header['CRVAL1'] *= spectral_scale
+            header['CUNIT1'] = self.unit.to_string(format='FITS')
+
+        return header
+
+
     @property
     def spectral_axis(self):
         """
         A `~astropy.units.Quantity` array containing the central values of
         each channel along the spectral axis.
         """
-        return self.wcs.wcs_pix2world(np.arange(self.size), 0)[0]
+
+        if self._wcs is None:
+            spec_axis = np.arange(self.size) * u.dimensionless_unscaled
+        else:
+            spec_axis = self.wcs.wcs_pix2world(np.arange(self.size), 0)[0] * \
+                u.Unit(self.wcs.wcs.cunit[0])
+            if self._spectral_unit is not None:
+                spec_axis = spec_axis.to(self._spectral_unit)
+
+        return spec_axis
 
     def quicklook(self, filename=None, drawstyle='steps-mid', **kwargs):
         """
@@ -244,3 +289,43 @@ class OneDSpectrum(LowerDimensionalObject):
         if filename is not None:
             pyplot.gcf().savefig(filename)
 
+
+    def with_spectral_unit(self, unit, velocity_convention=None,
+                           rest_value=None):
+
+        newwcs, newmeta = self._new_spectral_wcs(unit,
+                                                 velocity_convention=velocity_convention,
+                                                 rest_value=rest_value)
+
+        newheader = self._nowcs_header.copy()
+        newheader['CUNIT1'] = unit.to_string(format='FITS')
+
+
+        return OneDSpectrum(value=self.value, unit=self.unit, wcs=newwcs,
+                            header=newheader, meta=newmeta, copy=False,
+                            spectral_unit=unit)
+
+    def __getitem__(self, key, **kwargs):
+        try:
+            beams = self.beams[key]
+        except (AttributeError,TypeError):
+            beams = None
+
+        return super(OneDSpectrum, self).__getitem__(key, beams=beams)
+
+    @property
+    def hdu(self):
+        if hasattr(self, 'beams'):
+            warnings.warn("There are multiple beams for this spectrum that "
+                          "are being ignored when creating the HDU.")
+        return super(OneDSpectrum, self).hdu
+
+    @property
+    def hdulist(self):
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            hdu = self.hdu
+
+        beamhdu = beams_to_bintable(self.beams)
+
+        return HDUList([hdu, beamhdu])
