@@ -60,7 +60,7 @@ def dims_to_skip(shp1, shp2):
         # b==1 is broadcastable but not desired
         if a == 1:
             dims.append(len(shp2) - ii - 1)
-        elif  a == b:
+        elif a == b:
             pass
         else:
             raise ValueError("This should not be possible")
@@ -75,6 +75,10 @@ def view_of_subset(shp1, shp2, view):
     Given two shapes and a view, assuming that shape 1 can be broadcast
     to shape 2, return the sub-view that applies to shape 1
     """
+    # if the view is 1-dimensional, we can't subset it
+    if not hasattr(view, '__len__'):
+        return view
+
     dts = dims_to_skip(shp1, shp2)
     if view:
         cv_view = [x for ii,x in enumerate(view) if ii not in dts]
@@ -85,23 +89,24 @@ def view_of_subset(shp1, shp2, view):
 
     return cv_view
 
-
 class MaskBase(object):
 
     __metaclass__ = abc.ABCMeta
 
-    def include(self, data=None, wcs=None, view=()):
+    def include(self, data=None, wcs=None, view=(), **kwargs):
         """
         Return a boolean array indicating which values should be included.
 
         If ``view`` is passed, only the sliced mask will be returned, which
         avoids having to load the whole mask in memory. Otherwise, the whole
         mask is returned in-memory.
+
+        kwargs are passed to _validate_wcs
         """
-        self._validate_wcs(data, wcs)
+        self._validate_wcs(data, wcs, **kwargs)
         return self._include(data=data, wcs=wcs, view=view)
 
-    def _validate_wcs(self, data, wcs):
+    def _validate_wcs(self, new_data, new_wcs, **kwargs):
         """
         This method can be overridden in cases where the data and WCS have to
         conform to some rules. This gets called automatically when
@@ -113,15 +118,17 @@ class MaskBase(object):
     def _include(self, data=None, wcs=None, view=()):
         pass
 
-    def exclude(self, data=None, wcs=None, view=()):
+    def exclude(self, data=None, wcs=None, view=(), **kwargs):
         """
         Return a boolean array indicating which values should be excluded.
 
         If ``view`` is passed, only the sliced mask will be returned, which
         avoids having to load the whole mask in memory. Otherwise, the whole
         mask is returned in-memory.
+
+        kwargs are passed to _validate_wcs
         """
-        self._validate_wcs(data, wcs)
+        self._validate_wcs(data, wcs, **kwargs)
         return self._exclude(data=data, wcs=wcs, view=view)
 
     def _exclude(self, data=None, wcs=None, view=()):
@@ -149,7 +156,7 @@ class MaskBase(object):
         """
         return data[view][self.include(data=data, wcs=wcs, view=view)]
 
-    def _filled(self, data, wcs=None, fill=np.nan, view=()):
+    def _filled(self, data, wcs=None, fill=np.nan, view=(), **kwargs):
         """
         Replace the exluded elements of *array* with *fill*.
 
@@ -176,7 +183,7 @@ class MaskBase(object):
         # type otherwise
         dt = np.find_common_type([data.dtype], [np.float])
         sliced_data = data[view].astype(dt)
-        ex = self.exclude(data=data, wcs=wcs, view=view)
+        ex = self.exclude(data=data, wcs=wcs, view=view, **kwargs)
         sliced_data[ex] = fill
         return sliced_data
 
@@ -276,9 +283,9 @@ class CompositeMask(MaskBase):
         self._mask2 = mask2
         self._operation = operation
 
-    def _validate_wcs(self, new_data, new_wcs):
-        self._mask1._validate_wcs(new_data, new_wcs)
-        self._mask2._validate_wcs(new_data, new_wcs)
+    def _validate_wcs(self, new_data, new_wcs, **kwargs):
+        self._mask1._validate_wcs(new_data, new_wcs, **kwargs)
+        self._mask2._validate_wcs(new_data, new_wcs, **kwargs)
 
     def _include(self, data=None, wcs=None, view=()):
         result_mask_1 = self._mask1._include(data=data, wcs=wcs, view=view)
@@ -339,22 +346,65 @@ class BooleanArrayMask(MaskBase):
         if shape is not None and not is_broadcastable_and_smaller(mask.shape, shape):
             raise ValueError("Mask cannot be broadcast to the specified shape.")
         self._shape = shape or mask.shape
-        n_extra_dims = (len(self._shape)-mask.ndim)
-        if n_extra_dims > 0:
-            strides = (0,)*n_extra_dims + mask.strides
-            self._mask = as_strided(mask, shape=self.shape,
-                                    strides=strides)
-        else:
-            self._mask = mask
+        self._mask = mask
 
-    def _validate_wcs(self, new_data=None, new_wcs=None):
+        """
+        Developer note (AG):
+            The logic in this following section seems overly complicated.  All
+            of it is added to make sure that a 1D boolean array along the
+            spectral axis can be created.  I thought this was possible
+            previously, but experience many errors in my latest attempt to use
+            one.
+        """
+        # If a shape is given, we may need to broadcast to that shape
+        if shape is not None:
+            # these are dimensions that simply don't exist
+            n_empty_dims = (len(self._shape)-mask.ndim)
+
+            # these are dimensions of shape 1 that would be squeezed away but may
+            # be needed to make the arrays broadcastable (e.g., mask[:,None,None])
+            # Need to add n_empty_dims because (1,2) will broadcast to (3,1,2)
+            # and there will be no extra dims.
+            extra_dims = [ii
+                          for ii,(sh1,sh2) in
+                          enumerate(zip((0,)*n_empty_dims + mask.shape, shape))
+                          if sh1 == 1 and sh1 != sh2]
+
+
+            # Add the [None,]'s and the nonexistant
+            n_extra_dims =  n_empty_dims + len(extra_dims)
+
+            # if there are no extra dims, we're done, the original shape is fine
+            if n_extra_dims > 0:
+                strides = (0,)*n_empty_dims + mask.strides
+
+                for ed in extra_dims:
+                    # all of the [None,] dims should have 0 stride
+                    assert strides[ed] == 0,"Stride shape failure"
+
+                self._mask = as_strided(mask, shape=self.shape,
+                                        strides=strides)
+
+        # Make sure the mask shape matches the Mask object shape
+        assert self._mask.shape == self.shape,"Shape initialization failure"
+
+    def _validate_wcs(self, new_data=None, new_wcs=None, **kwargs):
+        """
+        Check that the new WCS matches the current one
+
+        Parameters
+        ----------
+        kwargs : dict
+            Passed to `wcs_utils.check_equality`
+        """
         if new_data is not None and not is_broadcastable_and_smaller(self._mask.shape,
                                                                      new_data.shape):
             raise ValueError("data shape cannot be broadcast to match mask shape")
         if new_wcs is not None:
             if new_wcs not in self._wcs_whitelist:
                 if not wcs_utils.check_equality(new_wcs, self._wcs,
-                                                warn_missing=True):
+                                                warn_missing=True,
+                                                **kwargs):
                     raise ValueError("WCS does not match mask WCS")
                 else:
                     self._wcs_whitelist.add(new_wcs)
@@ -373,7 +423,8 @@ class BooleanArrayMask(MaskBase):
 
     def __getitem__(self, view):
         return BooleanArrayMask(self._mask[view],
-                                wcs_utils.slice_wcs(self._wcs, view),
+                                wcs_utils.slice_wcs(self._wcs, view,
+                                                    shape=self.shape),
                                 shape=self._mask[view].shape)
 
     def with_spectral_unit(self, unit, velocity_convention=None, rest_value=None):
@@ -427,14 +478,22 @@ class LazyMask(MaskBase):
 
         self._wcs_whitelist = set()
 
-    def _validate_wcs(self, new_data=None, new_wcs=None):
+    def _validate_wcs(self, new_data=None, new_wcs=None, **kwargs):
+        """
+        Check that the new WCS matches the current one
+
+        Parameters
+        ----------
+        kwargs : dict
+            Passed to `wcs_utils.check_equality`
+        """
         if new_data is not None:
             if not is_broadcastable_and_smaller(new_data.shape, self._data.shape):
                 raise ValueError("data shape cannot be broadcast to match mask shape")
         if new_wcs is not None:
             if new_wcs not in self._wcs_whitelist:
                 if not wcs_utils.check_equality(new_wcs, self._wcs,
-                                                warn_missing=True):
+                                                warn_missing=True, **kwargs):
                     raise ValueError("WCS does not match mask WCS")
                 else:
                     self._wcs_whitelist.add(new_wcs)
@@ -445,7 +504,8 @@ class LazyMask(MaskBase):
 
     def __getitem__(self, view):
         return LazyMask(self._function, data=self._data[view],
-                        wcs=wcs_utils.slice_wcs(self._wcs, view))
+                        wcs=wcs_utils.slice_wcs(self._wcs, view,
+                                                shape=self._data.shape))
 
     def with_spectral_unit(self, unit, velocity_convention=None, rest_value=None):
         """
@@ -571,7 +631,7 @@ class FunctionMask(MaskBase):
     def __init__(self, function):
         self._function = function
 
-    def _validate_wcs(self, data, wcs):
+    def _validate_wcs(self, new_data, new_wcs, **kwargs):
         pass
 
     def _include(self, data=None, wcs=None, view=()):
